//*****************************************
// AVR ATmega644P Geiger Counter
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
// File Name:Ad_conv.c
//  Use A/D Covertor
// KCK[JE^[p
//   iq[Yf[^@is`fgpȂAȂAwIV[^(8MHz)Aɂj
//   i`FbN͂rohdmj
//
//  PA6(ADC6): Battery Monitor Max. about 1V (1V = 5V)
//  PA7(ADC7): High Volt Monitor Max. about 1V (1V = 1000V)
//
//  2011/09/02 New  Taishi ATmega644P(J308)
//  2011/10/22 Modify for LND712 by  Taishi
//  2011/10/28 Modify for SBM-20 by  Taishi
//  2011/11/08 Modify for LND712 by  Taishi
//  2011/11/12 Modify for SBM-20 by  Taishi
//  2011/11/12 Modify for J-308  by  Taishi
//  2011/11/24 Modify for All Model  Taishi
//  2012/03/31 Modify for All Model  Taishi(Ver 1.12) 
//*****************************************
#include				<avr/io.h>
#include				<avr/eeprom.h>

#include				"AVR_ATmega644_Geiger.h"
#include				"Utility.h"
#include				"Setting.h"
#include				"Display.h"
#include				"EEPROM.h"
#include				"Ad_conv.h"

unsigned int	ad7_data[N_of_AD_data];
unsigned char	ad7_count;
unsigned int	ad6_data[N_of_AD_data];
unsigned char	ad6_count;
unsigned int	hv_hosei_spn;
unsigned int	hv_hosei_ofs;
unsigned char	ad_conv_7;									// 0:ADC6 Convert Running, 1:ADC7 Convert running
// **********************************************************
// ************************* A/D 6 & 7***********************
// **********************************************************

//*****************************************
// A/D 6&7 Initial
// @荞ݕs̏ԂŌĂ΂Ƃ
//  PA6(ADC6): Batter Monitor Max. about 1V (1V = 5V)
//  PA7(ADC7): High Volt Monitor Max. about 1V (1V = 1000V)
//*****************************************
void	ad67_init(void)
{
	unsigned char	i;
	unsigned int	i_dim[3];

	for(i=0;i<3;i++){
		eeprom_busy_wait();
		i_dim[i] = eeprom_read_word(&eep_hv_hosei_spn[i]);								// gu@Xp␳l
	}

	hv_hosei_spn = tasuuketu_ui(i_dim[0], i_dim[1], i_dim[2], HV_HOSEI_SPN_DEFAULT);	// 

	if(2000 < hv_hosei_spn){
		hv_hosei_spn = HV_HOSEI_SPN_DEFAULT;
	}

	for(i=0;i<3;i++){
		if(hv_hosei_spn != i_dim[i]){
			eeprom_busy_wait();
			eeprom_write_word(&eep_hv_hosei_spn[i],hv_hosei_spn);						// gu@Xp␳l̕ۑ
		}
		eeprom_busy_wait();
	}

	for(i=0;i<3;i++){
		eeprom_busy_wait();
		i_dim[i] = eeprom_read_word(&eep_hv_hosei_ofs[i]);								// gu@ItZbg␳l
	}

	hv_hosei_ofs = tasuuketu_ui(i_dim[0], i_dim[1], i_dim[2], HV_HOSEI_OFS_DEFAULT);	// 

	if(1000 <= hv_hosei_ofs){
		hv_hosei_ofs = HV_HOSEI_OFS_DEFAULT;
	}

	for(i=0;i<3;i++){
		if(hv_hosei_ofs != i_dim[i]){
			eeprom_busy_wait();
			eeprom_write_word(&eep_hv_hosei_ofs[i],hv_hosei_ofs);						// gu@ItZbg␳l̕ۑ
		}
		eeprom_busy_wait();
	}

	ad6_count = 0;
	ad7_count = 0;
	for(i = 0; i < N_of_AD_data; i++){
		ad6_data[i] = 0;
		ad7_data[i] = 0;
	}

	DIDR0 |= ((1 << ADC6D) | (1 << ADC7D));													// AiO͐ݒ PD6,PD7

	ADMUX = ( (1 << REFS1) | (0 << REFS0) | (1 << MUX2) | (1 << MUX1) | (1 << MUX0) );		// PDPudgp, Select ADC7
	ADCSRA = ( (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) );	// A/D, A/DϊJn, CK/128=8MHz/128=62.5kHz(16uS) --> 208uS
	ad_conv_7 = 1;																			// 1:ADC7 Convert running

}

//*****************************************
// A/D 7 Set A/D data to Data Buffer
//  ϊIĂȂ́@Ȃ
//*****************************************
void	ad67_set_data(void)
{
	if( (ADCSRA & (1 << ADIF)) != 0){								// `^cϊ
		ADCSRA |= (1 << ADIF);										// `^cϊtO폜
		if( ad_conv_7 != 0){										// ADC7 ϊ
			ad7_data[ad7_count] = ADCW;
			ad7_count++;
			ad7_count &= (N_of_AD_data - 1);
			ADMUX &= ( (1 << REFS1) | (1 << REFS0) );				// dwrbg͎c
			ADMUX |= ( (1 << MUX2) | (1 << MUX1) | (0 << MUX0) );	// Select ADC6
			ad_conv_7 = 0;											// 0:ADC6 Convert Running
		}else{
			ad6_data[ad6_count] = ADCW;
			ad6_count++;
			ad6_count &= (N_of_AD_data - 1);
			ADMUX &= ( (1 << REFS1) | (1 << REFS0) );				// dwrbg͎c
			ADMUX |= ( (1 << MUX2) | (1 << MUX1) | (1 << MUX0) );	// Select ADC7
			ad_conv_7 = 1;											// 0:ADC6 Convert Running
		}

		ADCSRA |= (1 << ADSC);										// A/DϊJn
	}
}
//*****************************************
// A/D 7 Get A/D data
//  obt@ɓĂf[^𕽋ςĎ擾
//*****************************************
unsigned int	ad7_get_data(void)
{
	unsigned char	i;
	unsigned int	sum;

	sum = 0;
	for(i = 0; i < N_of_AD_data; i++){						// obt@Sđ
		sum += ad7_data[i];
	}

	sum /= N_of_AD_data;									// f[^Ŋĕϒlɂ

	return(sum);
}
//*****************************************
// A/D 6 Get A/D data
//  obt@ɓĂf[^𕽋ςĎ擾
//*****************************************
unsigned int	ad6_get_data(void)
{
	unsigned char	i;
	unsigned int	sum;

	sum = 0;
	for(i = 0; i < N_of_AD_data; i++){						// obt@Sđ
		sum += ad6_data[i];
	}

	sum /= N_of_AD_data;									// f[^Ŋĕϒlɂ

	return(sum);
}
